"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const ApiError_1 = __importDefault(require("./types/ApiError"));
const { version } = require("../package.json");
class ApiClient {
    constructor(config) {
        this.config = config;
    }
    get(requestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield axios_1.default(Object.assign({ method: "GET" }, this.buildRequestConfig(requestOptions)));
                return response.data;
            }
            catch (e) {
                throw this.buildError(e);
            }
        });
    }
    delete(requestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield axios_1.default(Object.assign({ method: "DELETE" }, this.buildRequestConfig(requestOptions)));
                return response.data;
            }
            catch (e) {
                throw this.buildError(e);
            }
        });
    }
    post(requestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield axios_1.default(Object.assign({ method: "POST" }, this.buildRequestConfig(requestOptions)));
                return response.data;
            }
            catch (e) {
                throw this.buildError(e);
            }
        });
    }
    put(requestOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield axios_1.default(Object.assign({ method: "PUT" }, this.buildRequestConfig(requestOptions)));
                return response.data;
            }
            catch (e) {
                throw this.buildError(e);
            }
        });
    }
    buildRequestConfig(requestOptions) {
        const config = Object.assign({ baseURL: this.config.baseUrl, headers: {
                Authorization: `ApiKey ${this.config.apiKey}`,
                'User-Agent': `warrant-node/${version}`,
            } }, requestOptions);
        if (requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.apiKey) {
            config.headers = {
                Authorization: `ApiKey ${requestOptions.apiKey}`,
                'User-Agent': `warrant-node/${version}`,
            };
        }
        if (requestOptions === null || requestOptions === void 0 ? void 0 : requestOptions.baseUrl) {
            config.baseURL = requestOptions.baseUrl;
        }
        return config;
    }
    buildError(e) {
        if (e.response) {
            return new ApiError_1.default(e.response.data.code, e.response.data.message);
        }
        return e;
    }
}
exports.default = ApiClient;
