"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
class Permission {
    constructor(permissionId, name, description) {
        this.permissionId = permissionId;
        this.name = name;
        this.description = description;
    }
    //
    // Static methods
    //
    static create(permission) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/permissions",
                    data: permission,
                });
                return new Permission(response.permissionId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/permissions/${permissionId}`,
                });
                return new Permission(response.permissionId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(permissionId, permission) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.put({
                    url: `/v1/permissions/${permissionId}`,
                    data: permission,
                });
                return new Permission(response.permissionId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/permissions/${permissionId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPermissions(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/permissions",
                    params: listOptions,
                });
                return response.map((permission) => new Permission(permission.permissionId, permission.name, permission.description));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPermissionsForUser(userId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}/permissions`,
                    params: listOptions,
                });
                return response.map((permission) => new Permission(permission.permissionId, permission.name, permission.description));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPermissionToUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: `/v1/users/${userId}/permissions/${permissionId}`,
                });
                return new Permission(response.permissionId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static removePermissionFromUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/users/${userId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPermissionsForRole(roleId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/roles/${roleId}/permissions`,
                    params: listOptions,
                });
                return response.map((permission) => new Permission(permission.permissionId, permission.name, permission.description));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPermissionToRole(roleId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: `/v1/roles/${roleId}/permissions/${permissionId}`,
                });
                return new Permission(response.permissionId, response.name, response.description);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static removePermissionFromRole(roleId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/roles/${roleId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Permission;
    }
    getObjectId() {
        return this.permissionId;
    }
}
exports.default = Permission;
